import {Component, OnDestroy, OnInit} from '@angular/core';
import {StudentInfoService} from 'src/app/services/student-info.service';
import {StudentDataService} from 'src/app/services/student-data.service';
import {QuestionDao} from 'src/app/dao/question.dao';
import {AnswerDao} from 'src/app/dao/answer.dao';
import {MinutePaperDao} from 'src/app/dao/minute-paper.dao';
import {StudentResponseDao} from 'src/app/dao/student-response.dao';
import {Router} from '@angular/router';
import {QuestionAnswerPairDao} from 'src/app/dao/question-answer-pair.dao';

@Component({
    selector: 'app-student-submission-view',
    templateUrl: './student-submission-view.component.html',
    styleUrls: ['./student-submission-view.component.css']
})
export class StudentSubmissionViewComponent implements OnInit, OnDestroy {

  private studentResponse: StudentResponseDao[];
  public currentPaper: MinutePaperDao;
  public studentId: String;
  public answerList: AnswerDao[];
  public questionList: QuestionDao[];
  public questionAnswerPairs: QuestionAnswerPairDao[];
  public invalidAnswers: Boolean;


  constructor(private studentInfoService: StudentInfoService,
              private studentDataService: StudentDataService,
              private router: Router) {
  }

  ngOnInit(): void {
    this.currentPaper = this.studentDataService.currentPaper;
    this.invalidAnswers = false;
    if (this.currentPaper) {
      this.studentId = sessionStorage.getItem('username');
      this.questionList = [];
      this.answerList = [];
      this.questionAnswerPairs = [];
      this.studentResponse = [];
      this.getQuestions();
      this.getAnswers();
    } else {
      this.router.navigate(['/sdashboard']);
      this.ngOnDestroy();
    }
  }

  ngOnDestroy(): void {
    this.questionList = [];
    this.answerList = [];
    this.studentResponse = [];
    this.currentPaper = undefined;
  }

  private populateAnswers(): void {
    let index = 0;
    this.questionAnswerPairs.forEach(q => {
        this.studentResponse.push(new StudentResponseDao(this.studentId, this.currentPaper.paperId,
          q.question.questionId, q.answer.answerText));
        index++;
    });
  }

  private organise() {
    this.questionAnswerPairs = [];
    for (let i = 0; i < this.questionList.length; i++) {
      if (this.questionList[i] && this.answerList[i]) {
        this.questionAnswerPairs.push(new QuestionAnswerPairDao(this.questionList[i], this.answerList[i]));
      } else if (this.questionList[i]) {
        this.questionAnswerPairs.push(new QuestionAnswerPairDao(this.questionList[i],
          new AnswerDao('0', '', this.studentId, this.currentPaper.paperId, this.questionList[i].questionId, false, '0000-00-00')));
      }
    }
  }

  private getQuestions() {
    this.studentInfoService.getPaperQuestions(this.studentId, this.currentPaper.paperId).subscribe((data: QuestionDao[]) => {
      data.forEach(e => this.questionList.push(e));
      this.organise();
    });
  }

  private getAnswers() {
    this.studentInfoService.getStudentAnswers(this.studentId, this.currentPaper.paperId).subscribe((data: AnswerDao[]) => {
      data.forEach(e => this.answerList.push(e));
      this.organise();
    });
  }

  private checkForEmpty(): Boolean {
    let result = false;
    this.questionAnswerPairs.forEach(a => {
      if (this.empty(a.answer.answerText)) {
        result = true;
      }
    });
    return result;
  }

  private checkForNonEmpty(): Boolean {
    let result = false;
    this.questionAnswerPairs.forEach(a => {
      if (!this.empty(a.answer.answerText)) {
        result = true;
      }
    });
    return result;
  }

  public empty(text: String): Boolean {
    if (text.replace(/\s/g, '').length === 0 || text === '') {
      return true;
    }
    return false;
  }

  public submit(): void {
    this.invalidAnswers = this.checkForEmpty();
    if (!this.invalidAnswers) {
      this.populateAnswers();
      this.studentInfoService.submitResponse(this.studentResponse).subscribe(() => {
        this.router.navigate(['/sdashboard']);
        this.ngOnDestroy();
      });
    }
  }

  public save(): void {
    this.populateAnswers();
    this.studentInfoService.saveDraft(this.studentResponse).subscribe(() => {
      this.router.navigate(['/sdashboard']);
      this.ngOnDestroy();
    });
  }

  public cancel(): void {
    if (this.checkForNonEmpty()) {
      const choice = confirm('Are you sure you want to cancel submission? All unsaved work will be lost!');
      if (choice) {
        this.router.navigate(['/sdashboard']);
        this.ngOnDestroy();
      }
    } else {
      this.router.navigate(['/sdashboard']);
      this.ngOnDestroy();
    }
  }
}
